package it.unimi.di.mg4j.index.cluster;

/*		 
 * MG4J: Managing Gigabytes for Java
 *
 * Copyright (C) 2006-2012 Sebastiano Vigna 
 *
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */

import it.unimi.di.mg4j.index.Index;
import it.unimi.di.mg4j.tool.URLMPHVirtualDocumentResolver;
import it.unimi.dsi.fastutil.ints.Int2IntOpenHashMap;
import it.unimi.dsi.fastutil.ints.IntArrays;
import it.unimi.dsi.fastutil.ints.IntOpenHashSet;
import it.unimi.dsi.fastutil.ints.IntSet;
import it.unimi.dsi.fastutil.io.BinIO;
import it.unimi.dsi.io.FastBufferedReader;
import it.unimi.dsi.lang.MutableString;
import it.unimi.dsi.util.Properties;

import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.net.URISyntaxException;

import org.apache.commons.configuration.ConfigurationException;

import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.Parameter;
import com.martiansoftware.jsap.SimpleJSAP;
import com.martiansoftware.jsap.UnflaggedOption;


/** A lexical strategy that creates an index containing a subset of the terms.
 * 
 * @author Sebastiano Vigna
 */


public class SubsetLexicalStrategy implements LexicalPartitioningStrategy {
	static final long serialVersionUID = 0;
	/** The local number of each term. */
	private final Int2IntOpenHashMap localNumber;
	
	/** Creates a new subset lexical strategy.
	 * @param subset the subset of terms.
	 */
	public SubsetLexicalStrategy( final IntSet subset ) {
		final int[] t = subset.toIntArray();
		IntArrays.quickSort( t );
		localNumber = new Int2IntOpenHashMap();
		localNumber.defaultReturnValue( -1 );
		for( int i = 0; i < t.length; i++ ) localNumber.put( t[ i ], i ); 
	}

	public int numberOfLocalIndices() {
		return 2;
	}

	public int localIndex( final int globalNumber ) {
		return localNumber.get( globalNumber ) == -1 ? 1 : 0;
	}

	public int localNumber( final int globalNumber ) {
		int n = localNumber.get( globalNumber );
		return n == -1 ? 0 : n;
	}

	@Override
	public Properties[] properties() {
		return null;
	}
	
	public static void main( final String[] arg ) throws JSAPException, IOException, ConfigurationException, SecurityException, URISyntaxException, ClassNotFoundException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		final SimpleJSAP jsap = new SimpleJSAP( URLMPHVirtualDocumentResolver.class.getName(), "Builds a subset-based lexical partitionin strategy reading an UTF-8 encoded term list from standard input.",
				new Parameter[] {
					new UnflaggedOption( "basename", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, JSAP.NOT_GREEDY, "The basename of the index." ),
					new UnflaggedOption( "strategy", JSAP.STRING_PARSER, JSAP.NO_DEFAULT, JSAP.REQUIRED, JSAP.NOT_GREEDY, "The filename for the strategy." )
		});
		
		JSAPResult jsapResult = jsap.parse( arg );
		if ( jsap.messagePrinted() ) return;
		
		final Index index = Index.getInstance( jsapResult.getString( "basename" ) );
		@SuppressWarnings("resource")
		final FastBufferedReader reader = new FastBufferedReader( new InputStreamReader( System.in, "UTF-8" ) );
		final MutableString s = new MutableString();
		final IntOpenHashSet subset = new IntOpenHashSet();

		while( reader.readLine( s ) != null ) {
			final int t = (int)index.termMap.getLong( s );
			if ( t == -1 ) throw new IllegalArgumentException( "Unknown term " + s );
			subset.add( t );
		}
		reader.close();

		BinIO.storeObject( new SubsetLexicalStrategy( subset ), jsapResult.getString( "strategy" ) );
	} 

}
